@@ -3,22 +3,24 @@ use anyhow::Result;
 
 use crc::{Crc, CRC_8_MAXIM_DOW};
 
+use tokio_serial::{SerialPort, SerialPortBuilderExt};
+
 pub const CRC_MAXIM: Crc<u8> = Crc::<u8>::new(&CRC_8_MAXIM_DOW);
 
-fn reset_device(device: &mut Box<dyn serialport::SerialPort>) -> Result<()> {
+async fn reset_device(device: &mut tokio_serial::SerialStream) -> Result<()> {
     device.write_request_to_send(false)?;
     device.write_data_terminal_ready(false)?;
-    std::thread::sleep(std::time::Duration::from_millis(250));
+    tokio::time::sleep(std::time::Duration::from_millis(250)).await;
     device.write_request_to_send(true)?;
     device.write_data_terminal_ready(true)?;
-    std::thread::sleep(std::time::Duration::from_millis(250));
+    tokio::time::sleep(std::time::Duration::from_millis(250)).await;
     Ok(())
 }
 
-fn flush_device<W: std::io::Write>(ser: &mut W) -> Result<()> {
+async fn flush_device<W: std::io::Write>(ser: &mut W) -> Result<()> {
     for _ in 0..5 {
         ser.flush()?;
-        std::thread::sleep(std::time::Duration::from_millis(50));
+        tokio::time::sleep(std::time::Duration::from_millis(50)).await;
     }
     Ok(())
 }
@@ -31,8 +33,8 @@ enum UdevError {
     Io(#[from] std::io::Error),
 }
 
-fn get_device_name(
-    device: &mut Box<dyn serialport::SerialPort>,
+async fn get_device_name(
+    device: &mut tokio_serial::SerialStream,
 ) -> std::result::Result<NameType, UdevError> {
     use std::io::{Read, Write};
     device.write_all(b"N?")?;
@@ -42,7 +44,7 @@ fn get_device_name(
     let mut buf = vec![0; alloc_len];
 
     // Wait half second for full answer.
-    std::thread::sleep(std::time::Duration::from_millis(500));
+    tokio::time::sleep(std::time::Duration::from_millis(500)).await;
 
     let len = device.read(&mut buf)?;
     assert!(len > DEVICE_NAME_LEN, "No CRC returned");
@@ -65,16 +67,25 @@ fn get_device_name(
     }
 }
 
-pub fn serial_handshake(port: &str) -> Result<(Box<dyn serialport::SerialPort>, NameType)> {
-    let mut ser = serialport::new(port, 9600)
-        .timeout(std::time::Duration::from_millis(500))
-        .open()?;
-    debug!("Resetting port {}", port);
-    reset_device(&mut ser)?;
-    std::thread::sleep(std::time::Duration::from_millis(2_500));
+pub async fn serial_handshake(
+    serial_device: &str,
+    baud_rate: u32,
+) -> Result<(tokio_serial::SerialStream, NameType)> {
+    #[allow(unused_mut)]
+    let mut ser = tokio_serial::new(serial_device, baud_rate).open_native_async()?;
+
+    #[cfg(unix)]
+    ser.set_exclusive(false)
+        .expect("Unable to set serial port exclusive to false");
+
+    debug!("Resetting port {}", serial_device);
+    reset_device(&mut ser).await?;
+    tokio::time::sleep(std::time::Duration::from_millis(2_500)).await;
     debug!("Flushing serial port");
-    flush_device(&mut ser)?;
+    flush_device(&mut ser).await?;
     debug!("Getting device name");
-    let name = get_device_name(&mut ser).map_err(anyhow::Error::from)?;
+    let name = get_device_name(&mut ser)
+        .await
+        .map_err(anyhow::Error::from)?;
     Ok((ser, name))
 }
